@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import typing
 import os
 import os.path
 import shutil
@@ -22,9 +23,11 @@ import logging
 import glob
 import threading
 import re
+from pathlib import Path
 
 from autokey import common
-from autokey.iomediator.constants import X_RECORD_INTERFACE
+from autokey.iomediator.constants import X_RECORD_INTERFACE, MODIFIERS
+from autokey.iomediator import key
 
 import json
 
@@ -48,7 +51,6 @@ PROMPT_TO_SAVE = "promptToSave"
 INPUT_SAVINGS = "inputSavings"
 ENABLE_QT4_WORKAROUND = "enableQT4Workaround"
 from .configmanager_constants import INTERFACE_TYPE
-# INTERFACE_TYPE = "interfaceType"
 UNDO_USING_BACKSPACE = "undoUsingBackspace"
 WINDOW_DEFAULT_SIZE = "windowDefaultSize"
 HPANE_POSITION = "hPanePosition"
@@ -56,6 +58,7 @@ COLUMN_WIDTHS = "columnWidths"
 SHOW_TOOLBAR = "showToolbar"
 NOTIFICATION_ICON = "notificationIcon"
 WORKAROUND_APP_REGEX = "workAroundApps"
+DISABLED_MODIFIERS = "disabledModifiers"
 # Added by Trey Blancher (ectospasm) 2015-09-16
 TRIGGER_BY_INITIAL = "triggerItemByInitial"
 
@@ -67,6 +70,11 @@ SCRIPT_GLOBALS = "scriptGlobals"
 #RECENT_ENTRY_MINLENGTH = "recentEntryMinLength"
 #RECENT_ENTRY_SUGGEST = "recentEntrySuggest"
 
+# Used to set or retrieve autostart related settings. These settings are separately handled by .desktop files in the
+# user autostart directory in $XDG_DATA_HOME/autostart, typically ~/.local/share/autostart.
+AutostartSettings = typing.NamedTuple("AutostartSettings", [
+    ("desktop_file_name", typing.Optional[str]), ("switch_show_configure", bool)
+])
 
 def get_config_manager(autoKeyApp, hadError=False):
     if not os.path.exists(CONFIG_DEFAULT_FOLDER):
@@ -119,14 +127,18 @@ def _persist_settings(config_manager):
     serializable_data = config_manager.get_serializable()
     try:
         _try_persist_settings(serializable_data)
-    except TypeError:
+    except (TypeError, ValueError):
         # The user added non-serializable data to the store, so remove all non-serializable keys or values.
         _remove_non_serializable_store_entries(serializable_data["settings"][SCRIPT_GLOBALS])
         _try_persist_settings(serializable_data)
 
 
 def _try_persist_settings(serializable_data: dict):
-    """Write the settings as JSON to the configuration file"""
+    """
+    Write the settings as JSON to the configuration file
+    :raises TypeError: If the user tries to store non-serializable types
+    :raises ValueError: If the user tries to store circular referenced (recursive) structures.
+    """
     with open(CONFIG_FILE, "w") as json_file:
             json.dump(serializable_data, json_file, indent=4)
 
@@ -140,9 +152,7 @@ def _remove_non_serializable_store_entries(store: dict):
     for key, value in store.items():
         if not (_is_serializable(key) and _is_serializable(value)):
             _logger.info("Remove non-serializable item from the global script store. Key: '{}', Value: '{}'. "
-                         "This item cannot be saved and therefore will be lost.".format(
-                            key, value
-            ))
+                         "This item cannot be saved and therefore will be lost.".format(key, value))
             removed_key_list.append(key)
     for key in removed_key_list:
         del store[key]
@@ -152,12 +162,120 @@ def _is_serializable(data):
     """Check, if data is json serializable."""
     try:
         json.dumps(data)
-    except TypeError:
+    except (TypeError, ValueError):
+        # TypeError occurs with non-serializable types (type, function, etc.)
+        # ValueError occurs when circular references are found. Example: `l=[]; l.append(l)`
         return False
     else:
         return True
 
 
+def get_autostart() -> AutostartSettings:
+    """Returns the autostart settings as read from the system."""
+    autostart_file = Path(common.AUTOSTART_DIR) / "autokey.desktop"
+    if not autostart_file.exists():
+        return AutostartSettings(None, False)
+    else:
+        return _extract_data_from_desktop_file(autostart_file)
+
+
+def _extract_data_from_desktop_file(desktop_file: Path) -> AutostartSettings:
+    with open(str(desktop_file), "r") as file:
+        for line in file.readlines():
+            line = line.rstrip("\n")
+            if line.startswith("Exec="):
+                program_name = line.split("=")[1].split(" ")[0]
+                return AutostartSettings(program_name + ".desktop", line.endswith("-c"))
+    raise ValueError("Autostart autokey.desktop file does not contain any Exec line. File: {}".format(desktop_file))
+
+
+def set_autostart_entry(autostart_data: AutostartSettings):
+    """
+    Activates or deactivates autostarting autokey during user login.
+    Autostart is handled by placing a .desktop file into '$XDG_CONFIG_HOME/autostart', typically '~/.config/autostart'
+    """
+    _logger.info("Save autostart settings: {}".format(autostart_data))
+    autostart_file = Path(common.AUTOSTART_DIR) / "autokey.desktop"
+    if autostart_data.desktop_file_name is None:  # Choosing None as the GUI signals deleting the entry.
+        delete_autostart_entry()
+    else:
+        autostart_file.parent.mkdir(exist_ok=True)  # make sure that the parent autostart directory exists.
+        _create_autostart_entry(autostart_data, autostart_file)
+
+
+def _create_autostart_entry(autostart_data: AutostartSettings, autostart_file: Path):
+    """Create an autostart .desktop file in the autostart directory, if possible."""
+    try:
+        source_desktop_file = get_source_desktop_file(autostart_data.desktop_file_name)
+    except FileNotFoundError:
+        _logger.exception("Failed to find a usable .desktop file! Unable to find: {}".format(
+            autostart_data.desktop_file_name))
+    else:
+        _logger.debug("Found source desktop file that will be placed into the autostart directory: {}".format(
+            source_desktop_file))
+        with open(str(source_desktop_file), "r") as opened_source_desktop_file:
+            desktop_file_content = opened_source_desktop_file.read()
+        desktop_file_content = "\n".join(_manage_autostart_desktop_file_launch_flags(
+            desktop_file_content, autostart_data.switch_show_configure
+        )) + "\n"
+        with open(str(autostart_file), "w", encoding="UTF-8") as opened_autostart_file:
+            opened_autostart_file.write(desktop_file_content)
+        _logger.debug("Written desktop file: {}".format(autostart_file))
+
+
+def delete_autostart_entry():
+    """Remove a present autostart entry. If none is found, nothing happens."""
+    autostart_file = Path(common.AUTOSTART_DIR) / "autokey.desktop"
+    if autostart_file.exists():
+        autostart_file.unlink()
+        _logger.info("Deleted old autostart entry: {}".format(autostart_file))
+
+
+def get_source_desktop_file(desktop_file_name: str) -> Path:
+    """
+    Try to get the source .desktop file with the given name.
+    :raises FileNotFoundError: If no desktop file was found in the searched directories.
+    """
+    possible_paths = (
+        # Copy from local installation. Also used if the user explicitely customized the launcher .desktop file.
+        Path(common.XDG_DATA_HOME) / "applications",
+        # Copy from system-wide installation
+        Path("/", "usr", "share", "applications"),
+        # Copy from git source tree. This will probably not work when used, because the application won’t be in the PATH
+        Path(__file__).parent.parent.parent / "config"
+    )
+    for possible_path in possible_paths:
+        desktop_file = possible_path / desktop_file_name
+        if desktop_file.exists():
+            return desktop_file
+    raise FileNotFoundError("Desktop file for autokey could not be found. Searched paths: {}".format(possible_paths))
+
+
+def _manage_autostart_desktop_file_launch_flags(desktop_file_content: str, show_configure: bool) -> typing.Iterable[str]:
+    """Iterate over the desktop file contents. Yields all lines except for the "Exec=" line verbatim. Modifies
+    the Exec line to include the user desired command line switches (currently only one implemented)."""
+    for line in desktop_file_content.splitlines(keepends=False):
+        if line.startswith("Exec="):
+            exec_line = _modify_exec_line(line, show_configure)
+            _logger.info("Used 'Exec' line in desktop file: {}".format(exec_line))
+            yield exec_line
+        else:
+            yield line
+
+
+def _modify_exec_line(line: str, show_configure: bool) -> str:
+    if show_configure:
+        if line.endswith(" -c"):
+            return line
+        else:
+            return line + " -c"
+    else:
+        if line.endswith(" -c"):
+            return line[:-3]
+        else:
+            return line
+
+
 def apply_settings(settings):
     """
     Allows new settings to be added without users having to lose all their configuration
@@ -208,6 +326,25 @@ def _convertFolder(folderData, parent):
             i.persist()
 
 
+def convert_rename_autostart_entries_for_v0_95_3():
+    """
+    In versions <= 0.95.2, the autostart option in autokey-gtk copied the default autokey-gtk.desktop file into
+    $XDG_CONFIG_DIR/autostart (with minor, unrelated modifications).
+    For versions >= 0.95.3, the autostart file is renamed to autokey.desktop. In 0.95.3, the autostart functionality
+    is implemented for autokey-qt. Thus, it becomes possible to have an autostart file for both GUIs in the autostart
+    directory simultaneously. Because of the singleton nature of autokey, this becomes an issue and race-conditions
+    determine which GUI starts first. To prevent this, both GUIs will share a single autokey.desktop autostart entry,
+    allowing only one GUI to be started during login. This allows for much simpler code.
+    """
+    old_autostart_file = Path(common.AUTOSTART_DIR) / "autokey-gtk.desktop"
+    if old_autostart_file.exists():
+        new_file_name = Path(common.AUTOSTART_DIR) / "autokey.desktop"
+        _logger.info("Migration task: Found old autostart entry: '{}'. Rename to: '{}'".format(
+            old_autostart_file, new_file_name)
+        )
+        old_autostart_file.rename(new_file_name)
+
+
 class ConfigManager:
     """
     Contains all application configuration, and provides methods for updating and 
@@ -239,6 +376,7 @@ class ConfigManager:
                 NOTIFICATION_ICON: common.ICON_FILE_NOTIFICATION,
                 WORKAROUND_APP_REGEX: ".*VirtualBox.*|krdc.Krdc",
                 TRIGGER_BY_INITIAL: False,
+                DISABLED_MODIFIERS: [],
                 # TODO - Future functionality
                 #TRACK_RECENT_ENTRY: True,
                 #RECENT_ENTRY_COUNT: 5,
@@ -264,10 +402,13 @@ class ConfigManager:
         
         self.toggleServiceHotkey = GlobalHotkey()
         self.toggleServiceHotkey.set_hotkey(["<super>", "<shift>"], "k")
-        self.toggleServiceHotkey.enabled = True    
-        
-        app.init_global_hotkeys(self)        
-        
+        self.toggleServiceHotkey.enabled = True
+
+        # Set the attribute to the default first. Without this, AK breaks, if started for the first time. See #274
+        self.workAroundApps = re.compile(self.SETTINGS[WORKAROUND_APP_REGEX])
+
+        app.init_global_hotkeys(self)
+
         self.load_global_config()
                 
         self.app.monitor.add_watch(CONFIG_DEFAULT_FOLDER)
@@ -404,10 +545,14 @@ dialog.info_dialog("Window information",
                     _logger.error("Existing config file has been saved as %s%s",
                                   CONFIG_FILE, version)
                     raise
+
+            if version < "0.95.3":
+                convert_rename_autostart_entries_for_v0_95_3()
                 
             self.VERSION = data["version"]
             self.userCodeDir = data["userCodeDir"]
             apply_settings(data["settings"])
+            self.load_disabled_modifiers()
             
             self.workAroundApps = re.compile(self.SETTINGS[WORKAROUND_APP_REGEX])
             
@@ -541,7 +686,75 @@ dialog.info_dialog("Window information",
         else:
             self.config_altered(False)
         return deleted
-            
+
+    def load_disabled_modifiers(self):
+        """
+        Load all disabled modifier keys from the configuration file. Called during startup, after the configuration
+        is read into the SETTINGS dictionary.
+        :return:
+        """
+        try:
+            self.SETTINGS[DISABLED_MODIFIERS] = [key.Key(value) for value in self.SETTINGS[DISABLED_MODIFIERS]]
+        except ValueError:
+            _logger.error("Unknown value in the disabled modifier list found. Unexpected: {}".format(
+                self.SETTINGS[DISABLED_MODIFIERS]))
+            self.SETTINGS[DISABLED_MODIFIERS] = []
+
+        for possible_modifier in self.SETTINGS[DISABLED_MODIFIERS]:
+            self._check_if_modifier(possible_modifier)
+            _logger.info("Disabling modifier key {} based on the stored configuration file.".format(possible_modifier))
+            MODIFIERS.remove(possible_modifier)
+
+    @staticmethod
+    def is_modifier_disabled(modifier: key.Key) -> bool:
+        """Checks, if the given modifier key is disabled. """
+        ConfigManager._check_if_modifier(modifier)
+        return modifier in ConfigManager.SETTINGS[DISABLED_MODIFIERS]
+
+    @staticmethod
+    def disable_modifier(modifier: typing.Union[key.Key, str]):
+        """
+        Permanently disable a modifier key. This can be used to disable unwanted modifier keys, like CAPSLOCK,
+        if the user remapped the physical key to something else.
+        :param modifier: Modifier key to disable.
+        :return:
+        """
+        if isinstance(modifier, str):
+            modifier = key.Key(modifier)
+        ConfigManager._check_if_modifier(modifier)
+        try:
+            _logger.info("Disabling modifier key {} on user request.".format(modifier))
+            MODIFIERS.remove(modifier)
+        except ValueError:
+            _logger.warning("Disabling already disabled modifier key. Affected key: {}".format(modifier))
+        else:
+            ConfigManager.SETTINGS[DISABLED_MODIFIERS].append(modifier)
+
+    @staticmethod
+    def enable_modifier(modifier: typing.Union[key.Key, str]):
+        """
+        Enable a previously disabled modifier key.
+        :param modifier: Modifier key to re-enable
+        :return:
+        """
+        if isinstance(modifier, str):
+            modifier = key.Key(modifier)
+        ConfigManager._check_if_modifier(modifier)
+        if modifier not in MODIFIERS:
+            _logger.info("Re-eabling modifier key {} on user request.".format(modifier))
+            MODIFIERS.append(modifier)
+            ConfigManager.SETTINGS[DISABLED_MODIFIERS].remove(modifier)
+        else:
+            _logger.warning("Enabling already enabled modifier key. Affected key: {}".format(modifier))
+
+    @staticmethod
+    def _check_if_modifier(modifier: key.Key):
+        if not isinstance(modifier, key.Key):
+            raise TypeError("The given value must be an AutoKey Key instance, got {}".format(type(modifier)))
+        if not modifier in key._ALL_MODIFIERS_:
+            raise ValueError("The given key '{}' is not a modifier. Expected one of {}.".format(
+                modifier, key._ALL_MODIFIERS_))
+
     def reload_global_config(self):
         _logger.info("Reloading global configuration")
         with open(CONFIG_FILE, 'r') as pFile:
