@@ -38,6 +38,8 @@ import Xlib.threaded as xlib_threaded
 # issues.
 del xlib_threaded
 
+from Xlib.error import ConnectionClosedError
+
 
 from . import common
 
@@ -57,6 +59,8 @@ else:
         HAS_ATSPI = False
     except ValueError:
         HAS_ATSPI = False
+    except SyntaxError:  # pyatspi 2.26 fails when used with Python 3.7
+        HAS_ATSPI = False
 
 from Xlib import X, XK, display, error
 try:
@@ -84,10 +88,8 @@ MASK_INDEXES = [
 CAPSLOCK_LEDMASK = 1<<0
 NUMLOCK_LEDMASK = 1<<1
 
-from typing import Union
 
-
-def str_or_bytes_to_bytes(x: Union[str, bytes, memoryview]) -> bytes:
+def str_or_bytes_to_bytes(x: typing.Union[str, bytes, memoryview]) -> bytes:
     if type(x) == bytes:
         # logger.info("using LiuLang's python3-xlib")
         return x
@@ -100,6 +102,10 @@ def str_or_bytes_to_bytes(x: Union[str, bytes, memoryview]) -> bytes:
     raise RuntimeError("x must be str or bytes or memoryview object, type(x)={}, repr(x)={}".format(type(x), repr(x)))
 
 
+# This tuple is used to return requested window properties.
+WindowInfo = typing.NamedTuple("WindowInfo", [("wm_title", str), ("wm_class", str)])
+
+
 class AbstractClipboard:
     """
     Abstract interface for clipboard interactions.
@@ -156,8 +162,12 @@ else:
         @text.setter
         def text(self, new_content: str):
             Gdk.threads_enter()
-            self._clipboard.set_text(new_content, -1)
-            Gdk.threads_leave()
+            try:
+                # This call might fail and raise an Exception.
+                # If it does, make sure to release the mutex and not deadlock AutoKey.
+                self._clipboard.set_text(new_content, -1)
+            finally:
+                Gdk.threads_leave()
 
         @property
         def selection(self):
@@ -169,8 +179,12 @@ else:
         @selection.setter
         def selection(self, new_content: str):
             Gdk.threads_enter()
-            self._selection.set_text(new_content, -1)
-            Gdk.threads_leave()
+            try:
+                # This call might fail and raise an Exception.
+                # If it does, make sure to release the mutex and not deadlock AutoKey.
+                self._selection.set_text(new_content, -1)
+            finally:
+                Gdk.threads_leave()
 
 
 class XInterfaceBase(threading.Thread):
@@ -184,8 +198,8 @@ class XInterfaceBase(threading.Thread):
         self.setName("XInterface-thread")
         self.mediator = mediator  # type: IoMediator
         self.app = app
-        self.lastChars = []  # QT4 Workaround
-        self.__enableQT4Workaround = False  # QT4 Workaround
+        self.lastChars = [] # QT4 Workaround
+        self.__enableQT4Workaround = False # QT4 Workaround
         self.shutdown = False
         
         # Event loop
@@ -312,15 +326,11 @@ class XInterfaceBase(threading.Thread):
             if attr[0].startswith("XK"):
                 if attr[1] == code:
                     logger.debug("Alt-Grid: %s, %s", attr[0], attr[1])
-        logger.debug(repr(self.localDisplay.keysym_to_keycodes(XK.XK_ISO_Level3_Shift)))
 
-        logger.debug("X Server Keymap")
+        logger.debug("X Server Keymap, listing unmapped keys.")
         for char in "\\|`1234567890-=~!@#$%^&*()qwertyuiop[]asdfghjkl;'zxcvbnm,./QWERTYUIOP{}ASDFGHJKL:\"ZXCVBNM<>?":
-            keyCodeList = self.localDisplay.keysym_to_keycodes(ord(char))
-            keyCodeList = list(keyCodeList)
-            if len(keyCodeList) > 0:
-                logger.debug("[%s]: %s", char, keyCodeList)
-            else:
+            keyCodeList = list(self.localDisplay.keysym_to_keycodes(ord(char)))
+            if not keyCodeList:
                 logger.debug("No mapping for [%s]", char)
                 
     def __needsMutterWorkaround(self, item):
@@ -372,12 +382,11 @@ class XInterfaceBase(threading.Thread):
             
         for window in children:
             try:
-                title = self.get_window_title(window, False)
-                klass = self.get_window_class(window, False)
+                window_info = self.get_window_info(window, False)
                 
-                if title or klass:
+                if window_info.wm_title or window_info.wm_class:
                     for item in hotkeys:
-                        if item.get_applicable_regex() is not None and item._should_trigger_window_title((title, klass)):
+                        if item.get_applicable_regex() is not None and item._should_trigger_window_title(window_info):
                             self.__grabHotkey(item.hotKey, item.modifiers, window)
                             self.__grabRecurse(item, window, False)
                         
@@ -417,12 +426,11 @@ class XInterfaceBase(threading.Thread):
             
         for window in children:
             try:
-                title = self.get_window_title(window, False)
-                klass = self.get_window_class(window, False)
+                window_info = self.get_window_info(window, False)
                 
-                if title or klass:
+                if window_info.wm_title or window_info.wm_class:
                     for item in hotkeys:
-                        if item.get_applicable_regex() is not None and item._should_trigger_window_title((title, klass)):
+                        if item.get_applicable_regex() is not None and item._should_trigger_window_title(window_info):
                             self.__ungrabHotkey(item.hotKey, item.modifiers, window)
                             self.__ungrabRecurse(item, window, False)
                         
@@ -438,10 +446,9 @@ class XInterfaceBase(threading.Thread):
         """
         c = self.app.configManager
         hotkeys = c.hotKeys + c.hotKeyFolders
-        title = self.get_window_title(window)
-        klass = self.get_window_class(window)
+        window_info = self.get_window_info(window)
         for item in hotkeys:
-            if item.get_applicable_regex() is not None and item._should_trigger_window_title((title, klass)):
+            if item.get_applicable_regex() is not None and item._should_trigger_window_title(window_info):
                 self.__enqueue(self.__grabHotkey, item.hotKey, item.modifiers, window)
             elif self.__needsMutterWorkaround(item):
                 self.__enqueue(self.__grabHotkey, item.hotKey, item.modifiers, window)
@@ -460,11 +467,13 @@ class XInterfaceBase(threading.Thread):
             window.grab_key(keycode, mask, True, X.GrabModeAsync, X.GrabModeAsync)
 
             if Key.NUMLOCK in self.modMasks:
-                mask |= self.modMasks[Key.NUMLOCK]
-            if Key.CAPSLOCK in self.modMasks:
-                mask |= self.modMasks[Key.CAPSLOCK]
+                window.grab_key(keycode, mask|self.modMasks[Key.NUMLOCK], True, X.GrabModeAsync, X.GrabModeAsync)
 
-            window.grab_key(keycode, mask, True, X.GrabModeAsync, X.GrabModeAsync)
+            if Key.CAPSLOCK in self.modMasks:
+                window.grab_key(keycode, mask|self.modMasks[Key.CAPSLOCK], True, X.GrabModeAsync, X.GrabModeAsync)
+
+            if Key.CAPSLOCK in self.modMasks and Key.NUMLOCK in self.modMasks:
+                window.grab_key(keycode, mask|self.modMasks[Key.CAPSLOCK]|self.modMasks[Key.NUMLOCK], True, X.GrabModeAsync, X.GrabModeAsync)
 
         except Exception as e:
             logger.warning("Failed to grab hotkey %r %r: %s", modifiers, key, str(e))
@@ -493,9 +502,8 @@ class XInterfaceBase(threading.Thread):
             shouldTrigger = False
             
             if checkWinInfo:
-                title = self.get_window_title(window, False)
-                klass = self.get_window_class(window, False)
-                shouldTrigger = item._should_trigger_window_title((title, klass))
+                window_info = self.get_window_info(window, False)
+                shouldTrigger = item._should_trigger_window_title(window_info)
 
             if shouldTrigger or not checkWinInfo:
                 self.__grabHotkey(item.hotKey, item.modifiers, window)
@@ -529,10 +537,9 @@ class XInterfaceBase(threading.Thread):
         for window in children:
             shouldTrigger = False
             
-            if checkWinInfo:        
-                title = self.get_window_title(window, False)
-                klass = self.get_window_class(window, False)
-                shouldTrigger = item._should_trigger_window_title((title, klass))
+            if checkWinInfo:
+                window_info = self.get_window_info(window, False)
+                shouldTrigger = item._should_trigger_window_title(window_info)
 
             if shouldTrigger or not checkWinInfo:
                 self.__ungrabHotkey(item.hotKey, item.modifiers, window)
@@ -606,11 +613,17 @@ class XInterfaceBase(threading.Thread):
         logger.debug("Sending via clipboard enqueued.")
 
     def _send_string_clipboard(self, string: str, paste_command: model.SendMode):
-        """Use the clipboard to send a string.
+        """
+        Use the clipboard to send a string.
         """
         backup = self.clipboard.text  # Keep a backup of current content, to restore the original afterwards.
+        if backup is None:
+            logger.warning("Tried to backup the X clipboard content, but got None instead of a string.")
         self.clipboard.text = string
-        self.mediator.send_string(paste_command.value)
+        try:
+            self.mediator.send_string(paste_command.value)
+        finally:
+            self.ungrab_keyboard()
         # Because send_string is queued, also enqueue the clipboard restore, to keep the proper action ordering.
         self.__enqueue(self._restore_clipboard_text, backup)
 
@@ -618,12 +631,14 @@ class XInterfaceBase(threading.Thread):
         """Restore the clipboard content."""
         # Pasting takes some time, so wait a bit before restoring the content. Otherwise the restore is done before
         # the pasting happens, causing the backup to be pasted instead of the desired clipboard content.
-        time.sleep(0.1)
-        self.clipboard.text = backup
+        time.sleep(0.2)
+        self.clipboard.text = backup if backup is not None else ""
 
     def _send_string_selection(self, string: str):
         """Use the mouse selection clipboard to send a string."""
         backup = self.clipboard.selection  # Keep a backup of current content, to restore the original afterwards.
+        if backup is None:
+            logger.warning("Tried to backup the X PRIMARY selection content, but got None instead of a string.")
         self.clipboard.selection = string
         self.__enqueue(self._paste_using_mouse_button_2)
         self.__enqueue(self._restore_clipboard_selection, backup)
@@ -636,7 +651,7 @@ class XInterfaceBase(threading.Thread):
         # Programmatically pressing the middle mouse button seems VERY slow, so wait rather long.
         # It might be a good idea to make this delay configurable. There might be systems that need even longer.
         time.sleep(1)
-        self.clipboard.selection = backup
+        self.clipboard.selection = backup if backup is not None else ""
 
     def _paste_using_mouse_button_2(self):
         """Paste using the mouse: Press the second mouse button, then release it again."""
@@ -906,6 +921,16 @@ class XInterfaceBase(threading.Thread):
 
                 if self.shutdown:
                     break
+            except ConnectionClosedError:
+                # Autokey does not properly exit on logout. It causes an infinite exception loop, accumulating stack
+                # traces along. This acts like a memory leak, filling the system RAM until it hits an OOM condition.
+                # TODO: implement a proper exit mechanic that gracefully exits AutoKey in this case.
+                # Maybe react to a dbus message that announces the session end, before the X server forcefully closes
+                # the connection.
+                # See https://github.com/autokey/autokey/issues/198 for details
+                logger.exception("__flushEvents: Connection to the X server closed. Forcefully exiting Autokey now.")
+                import os
+                os._exit(1)
             except Exception:
                 logger.exception("__flushEvents: Some exception occured:")
                 pass
@@ -921,7 +946,8 @@ class XInterfaceBase(threading.Thread):
         if modifier is not None:
             self.mediator.handle_modifier_down(modifier)
         else:
-            self.mediator.handle_keypress(keyCode, self.get_window_title(focus), self.get_window_class(focus))
+            window_info = self.get_window_info(focus)
+            self.mediator.handle_keypress(keyCode, window_info)
 
     def handle_keyrelease(self, keyCode):
         self.__enqueue(self.__handleKeyrelease, keyCode)
@@ -934,21 +960,23 @@ class XInterfaceBase(threading.Thread):
     def handle_mouseclick(self, button, x, y):
         self.__enqueue(self.__handleMouseclick, button, x, y)
         
-    def __handleMouseclick(self, button, x, y):        
-        title = self.get_window_title()
-        klass = self.get_window_class()
-        info = (title, klass)
+    def __handleMouseclick(self, button, x, y):
+        # Sleep a bit to timing issues. A mouse click might change the active application.
+        # If so, the switch happens asynchronously somewhere during the execution of the first two queries below,
+        # causing the queried window title (and maybe the window class or even none of those) to be invalid.
+        time.sleep(0.005)  # TODO: may need some tweaking
+        window_info = self.get_window_info()
         
         if x is None and y is None:
             ret = self.localDisplay.get_input_focus().focus.query_pointer()
-            self.mediator.handle_mouse_click(ret.root_x, ret.root_y, ret.win_x, ret.win_y, button, info)
+            self.mediator.handle_mouse_click(ret.root_x, ret.root_y, ret.win_x, ret.win_y, button, window_info)
         else:
             focus = self.localDisplay.get_input_focus().focus
             try:
                 rel = focus.translate_coords(self.rootWindow, x, y)
-                self.mediator.handle_mouse_click(x, y, rel.x, rel.y, button, info)
+                self.mediator.handle_mouse_click(x, y, rel.x, rel.y, button, window_info)
             except:
-                self.mediator.handle_mouse_click(x, y, 0, 0, button, info)
+                self.mediator.handle_mouse_click(x, y, 0, 0, button, window_info)
 
     def __decodeModifier(self, keyCode):
         """
@@ -969,10 +997,9 @@ class XInterfaceBase(threading.Thread):
 
     def __checkWorkaroundNeeded(self):
         focus = self.localDisplay.get_input_focus().focus
-        window_title = self.get_window_title(focus)
-        window_class = self.get_window_class(focus)
+        window_info = self.get_window_info(focus)
         w = self.app.configManager.workAroundApps
-        if w.match(window_title) or w.match(window_class):
+        if w.match(window_info.wm_title) or w.match(window_info.wm_class):
             self.__enableQT4Workaround = True
         else:
             self.__enableQT4Workaround = False
@@ -1040,67 +1067,99 @@ class XInterfaceBase(threading.Thread):
                 logger.error("Unknown key name: %s", char)
                 raise
 
-    def get_window_title(self, window=None, traverse=True) -> str:
+    def get_window_info(self, window=None, traverse: bool=True) -> WindowInfo:
         try:
             if window is None:
-                windowvar = self.localDisplay.get_input_focus().focus
-            else:
-                windowvar = window
+                window = self.localDisplay.get_input_focus().focus
+            return self._get_window_info(window, traverse)
+        except error.BadWindow:
+            logger.exception("Got BadWindow error while requesting window information.")
+            return self._create_window_info(window, "", "")
 
-            return self._get_window_title(windowvar, traverse)
+    def _get_window_info(self, window, traverse: bool, wm_title: str=None, wm_class: str=None) -> WindowInfo:
+        new_wm_title = self._try_get_window_title(window)
+        new_wm_class = self._try_get_window_class(window)
 
-        except AttributeError as e:
-            if str(e) == "'int' object has no attribute 'get_property'":
-                return ""
-            raise
-        except error.BadWindow as e:#TODO_PY3
-            logger.exception("Got BadWindow error.")
-            return ""
-        except:  # Default handler
-            return ""
+        if not wm_title and new_wm_title:  # Found title, update known information
+            wm_title = new_wm_title
+        if not wm_class and new_wm_class:  # Found class, update known information
+            wm_class = new_wm_class
 
-    def _get_window_title(self, windowvar, traverse) -> str:
-        atom = windowvar.get_property(self.__VisibleNameAtom, 0, 0, 255)
+        if traverse:
+            # Recursive operation on the parent window
+            if wm_title and wm_class:  # Both known, abort walking the tree and return the data.
+                return self._create_window_info(window, wm_title, wm_class)
+            else:  # At least one property is still not known. So walk the window tree up.
+                parent = window.query_tree().parent
+                # Stop traversal, if the parent is not a window. When querying the parent, at some point, an integer
+                # is returned. Then just stop following the tree.
+                if isinstance(parent, int):
+                    # At this point, wm_title or wm_class may still be None. The recursive call with traverse=False
+                    # will replace any None with an empty string. See below.
+                    return self._get_window_info(window, False, wm_title, wm_class)
+                else:
+                    return self._get_window_info(parent, traverse, wm_title, wm_class)
+
+        else:
+            # No recursion, so fill unknown values with empty strings.
+            if wm_title is None:
+                wm_title = ""
+            if wm_class is None:
+                wm_class = ""
+            return self._create_window_info(window, wm_title, wm_class)
+
+    def _create_window_info(self, window, wm_title: str, wm_class: str):
+        """
+        Creates a WindowInfo object from the window title and WM_CLASS.
+        Also checks for the Java XFocusProxyWindow workaround and applies it if needed:
+
+        Workaround for Java applications: Java AWT uses a XFocusProxyWindow class, so to get usable information,
+        the parent window needs to be queried. Credits: https://github.com/mooz/xkeysnail/pull/32
+        https://github.com/JetBrains/jdk8u_jdk/blob/master/src/solaris/classes/sun/awt/X11/XFocusProxyWindow.java#L35
+        """
+        if "FocusProxy" in wm_class:
+            parent = window.query_tree().parent
+            # Discard both the already known wm_class and window title, because both are known to be wrong.
+            return self._get_window_info(parent, False)
+        else:
+            return WindowInfo(wm_title=wm_title, wm_class=wm_class)
+
+    def _try_get_window_title(self, window) -> typing.Optional[str]:
+        atom = self._try_read_property(window, self.__VisibleNameAtom)
         if atom is None:
-            atom = windowvar.get_property(self.__NameAtom, 0, 0, 255)
+            atom = self._try_read_property(window, self.__NameAtom)
         if atom:
-            value = atom.value  # type: Union[str, bytes]
+            value = atom.value  # type: typing.Union[str, bytes]
             # based on python3-xlib version, atom.value may be a bytes object, then decoding is necessary.
             return value.decode("utf-8") if isinstance(value, bytes) else value
-        elif traverse:
-            return self._get_window_title(windowvar.query_tree().parent, True)
         else:
-            return ""
+            return None
+
+    @staticmethod
+    def _try_read_property(window, property_name: str):
+        """
+        Try to read the given property of the given window.
+        Returns the atom, if successful, None otherwise.
+        """
+        try:
+            return window.get_property(property_name, 0, 0, 255)
+        except error.BadAtom:
+            return None
+
+    @staticmethod
+    def _try_get_window_class(window) -> typing.Optional[str]:
+        wm_class = window.get_wm_class()
+        if wm_class:
+            return "{}.{}".format(wm_class[0], wm_class[1])
+        else:
+            return None
+
+    def get_window_title(self, window=None, traverse=True) -> str:
+        return self.get_window_info(window, traverse).wm_title
 
     def get_window_class(self, window=None, traverse=True) -> str:
-        try:
-            if window is None:
-                windowvar = self.localDisplay.get_input_focus().focus
-            else:
-                windowvar = window
+        return self.get_window_info(window, traverse).wm_class
 
-            return self._get_window_class(windowvar, traverse)
-        except AttributeError as e:
-            if str(e)=="'int' object has no attribute 'get_wm_class'":
-                return ""
-            raise
-        except error.BadWindow as e:#TODO_PY3
-            print(__name__, repr(e))
-            return ""
-        # except:
-        #     return ""
-    
-    def _get_window_class(self, windowvar, traverse) -> str:
-        wmclass = windowvar.get_wm_class()
-
-        if wmclass is None or not wmclass:
-            if traverse:
-                return self._get_window_class(windowvar.query_tree().parent, True)
-            else:
-                return ""
-
-        return wmclass[0] + '.' + wmclass[1]
-    
     def cancel(self):
         logger.debug("XInterfaceBase: Try to exit event thread.")
         self.queue.put_nowait((None, None))
